@@ -3,11 +3,12 @@
 import collections
 import hashlib
 import logging
-from typing import Any, Iterable, Optional, Tuple, Union
+from typing import Any, Iterable, Mapping, Optional, Sequence, Tuple, Union
 
 from pytype import compat
 from pytype import datatypes
 from pytype import utils
+from pytype.pyc import opcodes
 from pytype.pyc import pyc
 from pytype.pytd import mro
 from pytype.typegraph import cfg
@@ -15,8 +16,9 @@ from pytype.typegraph import cfg_utils
 
 log = logging.getLogger(__name__)
 
-# We can't import abstract.BaseValue here due to a circular dep.
-_BaseValue = Any
+# We can't import abstract here due to a circular dep.
+_BaseValue = Any  # abstract.BaseValue
+_TypeParameter = Any  # abstract.TypeParameter
 
 # Type parameter names matching the ones in builtins.pytd and typing.pytd.
 T = "_T"
@@ -655,7 +657,13 @@ def get_annotations_dict(members):
 class Local:
   """A possibly annotated local variable."""
 
-  def __init__(self, node, op, typ, orig, vm):
+  def __init__(
+      self,
+      node,
+      op: Optional[opcodes.Opcode],
+      typ: Optional[_BaseValue],
+      orig: Optional[cfg.Variable],
+      vm):
     self._ops = [op]
     if typ:
       self.typ = vm.program.NewVariable([typ], [], node)
@@ -666,15 +674,9 @@ class Local:
     self.orig = orig
     self.vm = vm
 
-  @property
-  def last_op(self):
-    # TODO(b/74434237): This property can be removed once the usage of it in
-    # dataclass_overlay is gone.
-    return self._ops[-1]
-
   @property
   def stack(self):
-    return self.vm.simple_stack(self.last_op)
+    return self.vm.simple_stack(self._ops[-1])
 
   def update(self, node, op, typ, orig):
     """Update this variable's annotation and/or value."""
@@ -792,3 +794,36 @@ def expand_type_parameter_instances(bindings: Iterable[cfg.Binding]):
         bindings = param_value.bindings + bindings
         continue
     yield b
+
+
+def get_type_parameter_substitutions(
+    val: _BaseValue, type_params: Iterable[_TypeParameter]
+) -> Mapping[str, cfg.Variable]:
+  return {p.full_name: val.get_instance_type_parameter(p.name)
+          for p in type_params}
+
+
+def build_generic_template(
+    type_params: Sequence[_BaseValue], base_type: _BaseValue
+) -> Tuple[Sequence[str], Sequence[_TypeParameter]]:
+  """Build a typing.Generic template from a sequence of type parameters."""
+  if not all(item.isinstance_TypeParameter() for item in type_params):
+    base_type.vm.errorlog.invalid_annotation(
+        base_type.vm.frames, base_type,
+        "Parameters to Generic[...] must all be type variables")
+    type_params = [item for item in type_params
+                   if item.isinstance_TypeParameter()]
+
+  template = [item.name for item in type_params]
+
+  if len(set(template)) != len(template):
+    base_type.vm.errorlog.invalid_annotation(
+        base_type.vm.frames, base_type,
+        "Parameters to Generic[...] must all be unique")
+
+  return template, type_params
+
+
+def is_generic_protocol(val: _BaseValue) -> bool:
+  return (val.isinstance_ParameterizedClass() and
+          val.full_name == "typing.Protocol")
