@@ -198,12 +198,53 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
     if errorlog:
       self.vm.errorlog.invalid_annotation(
           self.vm.frames, annot, details=errorlog.details)
+    if frame.func and isinstance(frame.func.data, abstract.BoundFunction):
+      self_var = frame.f_locals.pyval.get("self")
+      if self_var:
+        allowed_type_params = []
+        for v in self_var.data:
+          if v.cls:
+            allowed_type_params.extend(p.name for p in v.cls.template)
+      else:
+        allowed_type_params = ()
+    else:
+      self_var = None
+      allowed_type_params = self.vm.frame.type_params
     typ = self.extract_annotation(
-        state.node, var, name, self.vm.simple_stack(), is_var=True)
-    _, value = self.init_annotation(state.node, name, typ)
+        state.node, var, name, self.vm.simple_stack(),
+        allowed_type_params=allowed_type_params)
+    if typ.formal and self_var:
+      type_params = self.get_type_parameters(typ)
+      substs = [
+          abstract_utils.get_type_parameter_substitutions(v, type_params)
+          for v in self_var.data]
+      resolved_type = self.sub_one_annotation(state.node, typ, substs,
+                                              instantiate_unbound=False)
+      _, value = self.init_annotation(state.node, name, resolved_type)
+    elif typ.formal:
+      # The only time extract_annotation returns a formal type (i.e., one that
+      # contains type parameters) is when we are in the body of a generic class.
+      # Without `self_var`, we don't know what to instantiate those parameters
+      # to. We set the value to empty to defer instantiation until the value is
+      # being looked up on class instances, at which point the parameters will
+      # have been filled in.
+      value = self.vm.convert.empty.to_variable(state.node)
+    else:
+      _, value = self.init_annotation(state.node, name, typ)
     return typ, value
 
-  def extract_annotation(self, node, var, name, stack, is_var=False):
+  def extract_annotation(
+      self, node, var, name, stack, allowed_type_params=None):
+    """Returns an annotation extracted from 'var'.
+
+    Args:
+      node: The current node.
+      var: The variable to extract from.
+      name: The annotated name.
+      stack: The frame stack.
+      allowed_type_params: Type parameters that are allowed to appear in the
+        annotation. 'None' means all are allowed.
+    """
     try:
       typ = abstract_utils.get_atomic_value(var)
     except abstract_utils.ConversionError:
@@ -212,18 +253,32 @@ class AnnotationsUtil(utils.VirtualMachineWeakrefMixin):
     typ = self._process_one_annotation(node, typ, name, stack)
     if not typ:
       return self.vm.convert.unsolvable
-    if typ.formal and is_var:
-      if "AnyStr" in [x.name for x in self.get_type_parameters(typ)]:
-        if self.vm.PY2:
-          str_type = "typing.Text"
-        else:
-          str_type = "Union[str, bytes]"
-        details = f"Note: AnyStr is a TypeVar; use {str_type} for string types."
-      else:
-        details = None
-      self.vm.errorlog.not_supported_yet(
-          stack, "using type parameter in variable annotation", details=details)
-      return self.vm.convert.unsolvable
+    if typ.formal and allowed_type_params is not None:
+      illegal_params = [x.name for x in self.get_type_parameters(typ)
+                        if x.name not in allowed_type_params]
+      if illegal_params:
+        details = "TypeVar(s) %s not in scope" % ", ".join(
+            repr(p) for p in utils.unique_list(illegal_params))
+        if self.vm.frame.func:
+          method = self.vm.frame.func.data
+          if isinstance(method, abstract.BoundFunction):
+            class_name = method.name.rsplit(".", 1)[0]
+          else:
+            class_name = method.name
+          details += " for class %r" % class_name
+        if "AnyStr" in illegal_params:
+          if self.vm.PY2:
+            str_type = "typing.Text"
+          else:
+            str_type = "Union[str, bytes]"
+          details += (
+              f"\nNote: For all string types, use {str_type}.")
+        # TODO(b/186896951): Once the remaining use cases in the linked bug are
+        # supported, we should switch this to an [invalid-annotation] error.
+        self.vm.errorlog.not_supported_yet(
+            stack, "using type parameter in variable annotation",
+            details=details)
+        return self.vm.convert.unsolvable
     return typ
 
   def eval_multi_arg_annotation(self, node, func, annot, stack):
