@@ -275,15 +275,25 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
         # Fall back to __getattr__ if the attribute doesn't otherwise exist.
         node, attr = self._get_attribute_computed(
             node, cls, name, valself, compute_function="__getattr__")
-    if attr is None:
-      annots = abstract_utils.get_annotations_dict(obj.members)
-      if annots:
-        typ = annots.get_type(node, name)
-        if typ:
-          # An attribute has been declared but not defined, e.g.,
-          #   class Foo:
-          #     bar: int
-          _, attr = self.vm.annotations_util.init_annotation(node, name, typ)
+    if attr is None or attr.data == [self.vm.convert.empty]:
+      for base in obj.mro:
+        if not isinstance(base, abstract.InterpreterClass):
+          break
+        annots = abstract_utils.get_annotations_dict(base.members)
+        if annots:
+          typ = annots.get_type(node, name)
+          if typ:
+            # An attribute has been declared but not defined, e.g.,
+            #   class Foo:
+            #     bar: int
+            if typ.formal and valself:
+              subst = abstract_utils.get_type_parameter_substitutions(
+                  valself.data,
+                  self.vm.annotations_util.get_type_parameters(typ))
+              typ = self.vm.annotations_util.sub_one_annotation(
+                  node, typ, [subst])
+            _, attr = self.vm.annotations_util.init_annotation(node, name, typ)
+            break
     if attr is not None:
       attr = self._filter_var(node, attr)
     if attr is None and obj.maybe_missing_members:
@@ -350,10 +360,14 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
       # Deal with descriptors as a potential additional level of indirection.
       for v in attr.bindings:
         value = v.data
-        node2, getter = self.get_attribute(node, value, "__get__", v)
+        if (isinstance(value, special_builtins.PropertyInstance) and valself and
+            valself.data == cls):
+          node2, getter = node, None
+        else:
+          node2, getter = self.get_attribute(node, value, "__get__", v)
         if getter is not None:
           posargs = []
-          if valself:
+          if valself and valself.data != cls:
             posargs.append(valself.AssignToNewVariable())
           else:
             posargs.append(self.vm.convert.none.to_variable(node))
