@@ -11,7 +11,6 @@ from pytype import function
 from pytype import mixin
 from pytype.pytd import mro
 from pytype.pytd import pytd
-from pytype.pytd import visitors
 
 log = logging.getLogger(__name__)
 
@@ -66,24 +65,21 @@ class Attribute:
   pytd_const: Any = None
 
   @classmethod
-  def from_pytd_constant(cls, const, class_name, vm):
+  def from_pytd_constant(cls, const, vm, *, kw_only=False):
     """Generate an Attribute from a pytd.Constant."""
-    # We cannot yet handle recursive type annotations in pyi dataclasses, so set
-    # them to Any without raising an exception.
-    typ = const.type.Visit(visitors.ClassTypeToAny(class_name))  # pytype: disable=attribute-error
-    typ = vm.convert.constant_to_value(typ)
+    typ = vm.convert.constant_to_value(const.type)
     # We want to generate the default from the type, not from the value
     # (typically value will be Ellipsis or a similar placeholder).
     val = const.value and typ.instantiate(vm.root_node)
     # Dataclasses and similar decorators in pytd files cannot set init and
     # kw_only properties.
-    return cls(name=const.name, typ=typ, init=True, kw_only=False, default=val,
-               pytd_const=const)
+    return cls(name=const.name, typ=typ, init=True, kw_only=kw_only,
+               default=val, pytd_const=const)
 
   @classmethod
-  def from_param(cls, param, class_name, vm):
+  def from_param(cls, param, vm):
     const = pytd.Constant(param.name, param.type, param.optional)
-    return cls.from_pytd_constant(const, class_name, vm)
+    return cls.from_pytd_constant(const, vm, kw_only=param.kwonly)
 
   def to_pytd_constant(self):
     # TODO(mdemello): This is a bit fragile, but we only call this when
@@ -261,6 +257,10 @@ class Class(metaclass=mixin.MixinMeta):
     return any(base.full_name in ("unittest.TestCase", "unittest.case.TestCase")
                for base in self.mro)
 
+  @property
+  def is_enum(self):
+    return self.cls.full_name == "enum.EnumMeta" if self.cls else False
+
   @property
   def is_protocol(self):
     return bool(self.protocol_methods)
@@ -278,13 +278,18 @@ class Class(metaclass=mixin.MixinMeta):
     node, init = self.vm.attribute_handler.get_attribute(
         node, self.cls, "__init__")
     if not init or not any(
-        f.isinstance_InterpreterFunction() for f in init.data):
-      # Only an InterpreterFunction has interesting side effects.
+        f.isinstance_SignedFunction() for f in init.data):
+      # Only SignedFunctions (InterpreterFunction and SimpleFunction) have
+      # interesting side effects.
       return node
     # TODO(rechen): The signature is (cls, name, bases, dict); should we fill in
-    # the last three args more precisely?
-    args = function.Args(posargs=(self.to_variable(node),) + tuple(
-        self.vm.new_unsolvable(node) for _ in range(3)))
+    # the last arg more precisely?
+    args = function.Args(
+        posargs=(
+            self.to_variable(node),
+            self.vm.convert.build_string(node, self.name),
+            self.vm.convert.build_tuple(node, self.bases()),
+            self.vm.new_unsolvable(node)))
     log.debug("Calling __init__ on metaclass %s of class %s",
               self.cls.name, self.name)
     node, _ = self.vm.call_function(node, init, args)
@@ -292,13 +297,8 @@ class Class(metaclass=mixin.MixinMeta):
 
   def call_init_subclass(self, node):
     """Call init_subclass(cls) for all base classes."""
-    for b in self.bases():
-      # If a base has multiple bindings don't try to call init_subclass, since
-      # it is not clear what to do if different bindings implement the method
-      # differently.
-      if len(b.data) == 1:
-        base, = b.data
-        node = base.init_subclass(node, self)
+    for cls in self.mro:
+      node = cls.init_subclass(node, self)
     return node
 
   def get_own_new(self, node, value):
@@ -423,9 +423,17 @@ class Class(metaclass=mixin.MixinMeta):
     # calc MRO and replace them with original base classes
     return tuple(base2cls[base] for base in mro.MROMerge(newbases))
 
-  def _get_base_class_attrs(self, cls_attrs, metadata_key):
+  def _get_mro_attrs_for_attrs(self, cls_attrs, metadata_key):
     """Traverse the MRO and collect base class attributes for metadata_key."""
-    # We only add an attribute if it hasn't been defined before.
+    # For dataclasses, attributes preserve the ordering from the reversed MRO,
+    # but derived classes can override the type of an attribute. For attrs,
+    # derived attributes follow a more complicated scheme which we reproduce
+    # below.
+    #
+    # We take the dataclass behaviour as default, and special-case attrs.
+    #
+    # TODO(mdemello): See https://github.com/python-attrs/attrs/issues/428 -
+    # there are two separate behaviours, based on a `collect_by_mro` argument.
     base_attrs = []
     taken_attr_names = {a.name for a in cls_attrs}
     for base_cls in self.mro[1:]:
@@ -438,7 +446,24 @@ class Class(metaclass=mixin.MixinMeta):
         if a.name not in taken_attr_names:
           taken_attr_names.add(a.name)
           base_attrs.append(a)
-    return base_attrs
+    return base_attrs + cls_attrs
+
+  def _get_attrs_from_mro(self, cls_attrs, metadata_key):
+    """Traverse the MRO and collect base class attributes for metadata_key."""
+
+    if metadata_key == "__attrs_attrs__":
+      # attrs are special-cased
+      return self._get_mro_attrs_for_attrs(cls_attrs, metadata_key)
+
+    all_attrs = {}
+    sub_attrs = [base_cls.metadata.get(metadata_key, [])
+                 for base_cls in reversed(self.mro[1:])
+                 if isinstance(base_cls, Class)]
+    sub_attrs.append(cls_attrs)
+    for attrs in sub_attrs:
+      for a in attrs:
+        all_attrs[a.name] = a
+    return list(all_attrs.values())
 
   def record_attr_ordering(self, own_attrs):
     """Records the order of attrs to write in the output pyi."""
@@ -457,8 +482,7 @@ class Class(metaclass=mixin.MixinMeta):
     # We want this to crash if 'decorator' is not in _METADATA_KEYS
     assert decorator in _METADATA_KEYS, f"No metadata key for {decorator}"
     key = _METADATA_KEYS[decorator]
-    base_attrs = self._get_base_class_attrs(own_attrs, key)
-    attrs = base_attrs + own_attrs
+    attrs = self._get_attrs_from_mro(own_attrs, key)
     # Stash attributes in class metadata for subclasses.
     self.metadata[key] = attrs
     return attrs
