@@ -159,6 +159,11 @@ def add_basic_options(o):
       dest="attribute_variable_annotations", default=False,
       help=("Apply PEP 526-style variable annotations on attributes. " +
             temporary))
+  o.add_argument(
+      "--bind-properties", action="store_true",
+      dest="bind_properties", default=False,
+      help=("Bind @property methods to the classes they're defined on for more "
+            "precise type-checking. " + temporary))
   o.add_argument(
       "--check-parameter-types", action="store_true",
       dest="check_parameter_types", default=False,
@@ -172,6 +177,10 @@ def add_basic_options(o):
       "--preserve-union-macros", action="store_true",
       dest="preserve_union_macros", default=False,
       help="Preserve Union generic type aliases in pyi files. " + temporary)
+  o.add_argument(
+      "--use-enum-overlay", action="store_true",
+      dest="use_enum_overlay", default=False,
+      help="Use the enum overlay for more precise enum checking. " + temporary)
 
 
 def add_subtools(o):
