@@ -10,6 +10,7 @@ from pytype import class_mixin
 from pytype import compat
 from pytype import datatypes
 from pytype import function
+from pytype import overlay_dict
 from pytype import mixin
 from pytype import output
 from pytype import special_builtins
@@ -46,6 +47,10 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     self.vm.convert = self  # to make constant_to_value calls below work
     self.pytd_convert = output.Converter(vm)
 
+    # If set, allow construction of recursive values, setting the
+    # self-referential field to Any
+    self.recursion_allowed = False
+
     self._convert_cache = {}
     self._resolved_late_types = {}  # performance cache
 
@@ -239,12 +244,16 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     content = [var.AssignToNewVariable(node) for var in content]
     return abstract.List(content, self.vm).to_variable(node)
 
-  def build_list_of_type(self, node, var):
-    """Create a VM list with element type derived from the given variable."""
-    ret = abstract.Instance(self.list_type, self.vm)
+  def build_collection_of_type(self, node, typ, var):
+    """Create a collection Typ[T] with T derived from the given variable."""
+    ret = abstract.Instance(typ, self.vm)
     ret.merge_instance_type_parameter(node, abstract_utils.T, var)
     return ret.to_variable(node)
 
+  def build_list_of_type(self, node, var):
+    """Create a VM list with element type derived from the given variable."""
+    return self.build_collection_of_type(node, self.list_type, var)
+
   def build_set(self, node, content):
     """Create a VM set from the given sequence."""
     content = list(content)  # content might be a generator
@@ -386,7 +395,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     if isinstance(pyval, pytd.NothingType):
       return self.vm.program.NewVariable([], [], self.vm.root_node)
     elif isinstance(pyval, pytd.Alias):
-      return self.constant_to_var(pyval.type, subst, node, source_sets,  # pytype: disable=attribute-error
+      return self.constant_to_var(pyval.type, subst, node, source_sets,
                                   discard_concrete_values)
     elif isinstance(pyval, abstract_utils.AsInstance):
       cls = pyval.cls
@@ -459,10 +468,11 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     key = ("constant", pyval, type_key)
     if key in self._convert_cache:
       if self._convert_cache[key] is None:
-        # This error is triggered by, e.g., classes inheriting from each other.
-        name = getattr(pyval, "name", None) or pyval.__class__.__name__
-        self.vm.errorlog.recursion_error(self.vm.frames, name)
         self._convert_cache[key] = self.unsolvable
+        # This error is triggered by, e.g., classes inheriting from each other.
+        if not self.recursion_allowed:
+          name = getattr(pyval, "name", None) or pyval.__class__.__name__
+          self.vm.errorlog.recursion_error(self.vm.frames, name)
       return self._convert_cache[key]
     else:
       self._convert_cache[key] = None  # for recursion detection
@@ -610,8 +620,9 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       else:
         module, dot, base_name = pyval.name.rpartition(".")
         # typing.TypingContainer intentionally loads the underlying pytd types.
-        if module != "typing" and module in self.vm.loaded_overlays:
-          overlay = self.vm.loaded_overlays[module]
+        if (module not in ("typing", "typing_extensions") and
+            module in overlay_dict.overlays):
+          overlay = self.vm.import_module(module, module, 0)
           if overlay.get_module(base_name) is overlay:
             overlay.load_lazy_attribute(base_name)
             return abstract_utils.get_atomic_value(overlay.members[base_name])
@@ -623,6 +634,7 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         else:
           if dot:
             cls.module = module
+          cls.call_metaclass_init(get_node())
         return cls
     elif isinstance(pyval, pytd.Function):
       signatures = [function.PyTDSignature(pyval.name, sig, self.vm)
@@ -636,8 +648,15 @@ class Converter(utils.VirtualMachineWeakrefMixin):
       f.is_abstract = pyval.is_abstract
       return f
     elif isinstance(pyval, pytd.ClassType):
-      assert pyval.cls
-      return self.constant_to_value(pyval.cls, subst, self.vm.root_node)
+      if pyval.cls:
+        cls = pyval.cls
+      else:
+        # If pyval is a reference to a class in builtins or typing, we can fill
+        # in the class ourselves. lookup_builtin raises a KeyError if the name
+        # is not found.
+        cls = self.vm.lookup_builtin(pyval.name)
+        assert isinstance(cls, pytd.Class)
+      return self.constant_to_value(cls, subst, self.vm.root_node)
     elif isinstance(pyval, pytd.NothingType):
       return self.empty
     elif isinstance(pyval, pytd.AnythingType):
@@ -792,7 +811,8 @@ class Converter(utils.VirtualMachineWeakrefMixin):
           pyval_template = base.template
         template = tuple(t.name for t in pyval_template)
         parameters = pyval.parameters
-      assert pyval.name == "typing.Generic" or len(parameters) <= len(template)
+      assert (pyval.name in ("typing.Generic", "typing.Protocol") or
+              len(parameters) <= len(template))
       # Delay type parameter loading to handle recursive types.
       # See the ParameterizedClass.formal_type_parameters() property.
       type_parameters = abstract_utils.LazyFormalTypeParameters(
