@@ -1,10 +1,5 @@
 """Load and link .pyi files."""
 
-# TODO(b/175443166): pytype cannot infer the type of Loader._modules correctly.
-# Once strict attribute checking is rolled out, we should be able to annotate
-# _modules and remove this disable.
-# pytype: disable=attribute-error
-
 import collections
 import logging
 import os
@@ -135,7 +130,8 @@ class _ModuleMap:
 
   def __init__(self, python_version, modules=None):
     self.python_version = python_version
-    self._modules = modules or self._base_modules(self.python_version)
+    self._modules: Dict[str, Module] = modules or self._base_modules(
+        self.python_version)
     if self._modules["builtins"].needs_unpickling():
       self._unpickle_module(self._modules["builtins"])
     if self._modules["typing"].needs_unpickling():
@@ -572,8 +568,8 @@ class Loader:
             # local reference and not an import at all.
             continue
           else:
-            error = "Can't find pyi for %r" % name
-            raise BadDependencyError(error, ast_name)
+            self._path_finder.log_module_not_found(name)
+            raise BadDependencyError("Can't find pyi for %r" % name, ast_name)
       # If `name` is a package, try to load any base names not defined in
       # __init__ as submodules.
       if not self._modules[name].is_package() or "__getattr__" in dep_ast:
@@ -592,9 +588,10 @@ class Loader:
         # Alias(submodule, NamedType(submodule)).
         if attr is None or (
             isinstance(attr, pytd.Alias) and attr.name == attr.type.name):
-          # Don't check the import result - resolve_external_types will raise
-          # a better error.
-          self._import_module_by_name(full_name)
+          if not self._import_module_by_name(full_name):
+            # Add logging to make debugging easier but otherwise ignore the
+            # result - resolve_external_types will raise a better error.
+            self._path_finder.log_module_not_found(full_name)
 
   def _resolve_external_types(self, mod_ast, mod_name=None):
     module_map = self._modules.get_module_map()
@@ -667,6 +664,8 @@ class Loader:
     if module_name in self._import_name_cache:
       return self._import_name_cache[module_name]
     mod_ast = self._import_module_by_name(module_name)
+    if not mod_ast:
+      self._path_finder.log_module_not_found(module_name)
     self._resolve_classtype_pointers_for_all_modules()
     mod_ast = self.finish_and_verify_ast(mod_ast)
     self._import_name_cache[module_name] = mod_ast
@@ -766,8 +765,6 @@ class Loader:
       self._modules[module_name] = default
       return file_ast
 
-    # Add debug logging if we have not found the module.
-    self._path_finder.log_module_not_found(module_name)
     return None
 
   def _import_file(self, module_name):
