@@ -4,6 +4,7 @@ import collections
 import contextlib
 import enum
 import logging
+from typing import cast
 
 from pytype import abstract
 from pytype import abstract_utils
@@ -623,7 +624,8 @@ class Converter(utils.VirtualMachineWeakrefMixin):
 
     # class-level attributes
     for name, member in v.members.items():
-      if name in CLASS_LEVEL_IGNORE or name in annotated_names:
+      if (name in CLASS_LEVEL_IGNORE or name in annotated_names or
+          (v.is_enum and name == "__new__")):
         continue
       for value in member.FilteredData(self.vm.exitpoint, strict=False):
         if isinstance(value, special_builtins.PropertyInstance):
@@ -640,13 +642,41 @@ class Converter(utils.VirtualMachineWeakrefMixin):
         elif isinstance(value, special_builtins.ClassMethodInstance):
           add_decorated_method(name, value, pytd.MethodTypes.CLASSMETHOD)
         elif isinstance(value, abstract.Function):
+          # value_to_pytd_def returns different pytd node types depending on the
+          # input type, which pytype struggles to reason about.
+          method = cast(pytd.Function,
+                        self.value_to_pytd_def(node, value, name))
+          keep = lambda name: not name or name.startswith(v.name)
+          signatures = tuple(s for s in method.signatures
+                             if not s.params or keep(s.params[0].type.name))
+          if signatures and signatures != method.signatures:
+            # Filter out calls made from subclasses unless they are the only
+            # ones recorded; when inferring types for ParentClass.__init__, we
+            # do not want `self: Union[ParentClass, Subclass]`.
+            method = method.Replace(signatures=signatures)
           # TODO(rechen): Removing mutations altogether won't work for generic
           # classes. To support those, we'll need to change the mutated type's
           # base to the current class, rename aliased type parameters, and
           # replace any parameter not in the class or function template with
           # its upper value.
-          methods[name] = self.value_to_pytd_def(node, value, name).Visit(
-              visitors.DropMutableParameters())
+          methods[name] = method.Visit(visitors.DropMutableParameters())
+        elif v.is_enum and any(isinstance(enum_member, abstract.Instance)
+                               and enum_member.cls == v
+                               for enum_member in member.data):
+          # i.e. if this is an enum that has any enum members, and the current
+          # member is an enum member.
+          # In this case, we would normally output:
+          # class M(enum.Enum):
+          #   A: M
+          # However, this loses the type of A.value. Instead, annotate members
+          # with the type of their value. (This is what typeshed does.)
+          # class M(enum.Enum):
+          #   A: int
+          enum_member = abstract_utils.get_atomic_value(member)
+          node, attr_var = self.vm.attribute_handler.get_attribute(
+              node, enum_member, "value")
+          attr = abstract_utils.get_atomic_value(attr_var)
+          constants[name].add_type(attr.to_type(node))
         else:
           cls = self.vm.convert.merge_classes([value])
           node, attr = self.vm.attribute_handler.get_attribute(
@@ -664,6 +694,9 @@ class Converter(utils.VirtualMachineWeakrefMixin):
     # non-canonical instances are added if their canonical values do not contain
     # type parameters.
     ignore = set(annotated_names)
+    # enums should not print "name" and "value" for instances.
+    if v.is_enum:
+      ignore.update(("name", "value"))
     canonical_attributes = set()
 
     def add_attributes_from(instance):
