@@ -15,6 +15,7 @@ from pytype import abstract
 from pytype import abstract_utils
 from pytype import function
 from pytype import overlay
+from pytype.overlays import classgen
 from pytype.overlays import dataclass_overlay
 from pytype.pytd import pytd
 
@@ -37,6 +38,12 @@ class Dataclass(dataclass_overlay.Dataclass):
   def make(cls, vm):
     return super().make(vm, "flax.struct")
 
+  def decorate(self, node, cls):
+    super().decorate(node, cls)
+    if not isinstance(cls, abstract.InterpreterClass):
+      return
+    cls.members["replace"] = classgen.make_replace_method(self.vm, node, cls)
+
 
 # NOTE: flax.linen.module.Module is reexported as flax.linen.Module in
 # flax.linen/__init__.py. Due to the way the import system interacts with
@@ -93,10 +100,18 @@ class ModuleDataclass(dataclass_overlay.Dataclass):
     self._add_implicit_field(node, cls_locals, "parent", parent_type)
     return cls_locals
 
+  def decorate(self, node, cls):
+    super().decorate(node, cls)
+    if not isinstance(cls, abstract.InterpreterClass):
+      return
+    cls.members["replace"] = classgen.make_replace_method(self.vm, node, cls)
+
 
 class Module(abstract.PyTDClass):
   """Construct a dataclass for any class inheriting from Module."""
 
+  IMPLICIT_FIELDS = ("name", "parent")
+
   def __init__(self, vm, name="Module", module="flax.linen.module"):
     ast = vm.loader.import_name(module)
     pytd_cls = ast.Lookup(f"{module}.{name}")
