@@ -1,8 +1,5 @@
 """Opcode definitions for both Python 2 and Python 3."""
 
-# Because pytype takes too long:
-# pytype: skip-file
-
 import six
 import six.moves
 
@@ -28,7 +25,7 @@ class Opcode:
   """An opcode without arguments."""
 
   __slots__ = ("line", "index", "prev", "next", "target", "block_target",
-               "code", "annotation")
+               "code", "annotation", "folded")
   FLAGS = 0
 
   def __init__(self, index, line):
@@ -37,6 +34,7 @@ class Opcode:
     self.target = None
     self.code = None  # If we have a CodeType or OrderedCode parent
     self.annotation = None
+    self.folded = False  # elided by constant folding
 
   def at_line(self, line):
     """Return a new opcode simliar to this one but with a different line."""
@@ -48,7 +46,9 @@ class Opcode:
 
   def basic_str(self):
     """Helper function for the various __str__ formats."""
-    return "%d: %d: %s" % (self.line, self.index, self.__class__.__name__)
+    folded = "<<<<" if self.folded else ""
+    return "%d: %d: %s %s" % (
+        self.line, self.index, self.__class__.__name__, folded)
 
   def __str__(self):
     if self.annotation:
@@ -913,6 +913,11 @@ class DICT_UPDATE(OpcodeWithArg):
   __slots__ = ()
 
 
+class LOAD_FOLDED_CONST(OpcodeWithArg):  # A fake opcode used internally
+  FLAGS = HAS_ARGUMENT
+  __slots__ = ()
+
+
 python2_mapping = {
     0: STOP_CODE,  # removed in Python 3
     1: POP_TOP,
@@ -1370,8 +1375,7 @@ def _dis(python_version, data, mapping, reader,
         oparg += end_pos
       pretty = _prettyprint_arg(cls, oparg, co_consts, co_names, co_varnames,
                                 cellvars_freevars)
-      code.append(
-          cls(index, line, oparg, pretty))  # pytype: disable=wrong-arg-count
+      code.append(cls(index, line, oparg, pretty))
     else:
       code.append(cls(index, line))
 
