@@ -213,6 +213,38 @@ class ParserTest(_ParserTestBase):
           def c(cls) -> str: ...
     """)
 
+  def test_chained_assignment(self):
+    self.check("""
+      a = b = int
+    """, """
+      a = int
+      b = int
+    """)
+
+  def test_multiple_assignment(self):
+    self.check("""
+      a, b = int, str
+    """, """
+      a = int
+      b = str
+    """)
+    self.check("""
+      (a, b) = (c, d) = int, str
+    """, """
+      a = int
+      b = str
+      c = int
+      d = str
+    """)
+
+  def test_invalid_multiple_assignment(self):
+    self.check_error("""
+      a, b = int, str, bool
+    """, 1, "Cannot unpack 2 values for multiple assignment")
+    self.check_error("""
+      a, b = int
+    """, 1, "Cannot unpack 2 values for multiple assignment")
+
   def test_slots(self):
     self.check("""
       class A:
@@ -1307,6 +1339,22 @@ class FunctionTest(_ParserTestBase):
       def foo(x) -> int:
           y = int""", 1, "No parameter named y")
 
+  def test_mutator_from_annotation(self):
+    self.check("""
+      from typing import Generic, TypeVar
+      T = TypeVar('T')
+      class Foo(Generic[T]):
+          def __init__(self: Foo[str]) -> None: ...
+    """, """
+      from typing import Generic, TypeVar
+
+      T = TypeVar('T')
+
+      class Foo(Generic[T]):
+          def __init__(self) -> None:
+              self = Foo[str]
+    """)
+
   def test_exceptions(self):
     self.check("""
       def foo(x) -> int:
@@ -2909,5 +2957,50 @@ class ConcatenateTest(_ParserTestBase):
     """)
 
 
+class UnionOrTest(_ParserTestBase):
+
+  def test_basic(self):
+    self.check("""
+      def f(x: int | str) -> None: ...
+      def g(x: bool | str | float) -> None: ...
+      def h(x: str | None) -> None: ...
+    """, """
+      from typing import Optional, Union
+
+      def f(x: Union[int, str]) -> None: ...
+      def g(x: Union[bool, str, float]) -> None: ...
+      def h(x: Optional[str]) -> None: ...
+    """)
+
+
+class TypeGuardTest(_ParserTestBase):
+
+  def test_typing_extensions(self):
+    self.check("""
+      from typing import List
+
+      from typing_extensions import TypeGuard
+
+      def f(x: List[object]) -> TypeGuard[List[str]]: ...
+    """, """
+      from typing import List
+
+      from typing_extensions import TypeGuard
+
+      def f(x: List[object]) -> bool: ...
+  """)
+
+  def test_typing(self):
+    self.check("""
+      from typing import List, TypeGuard
+
+      def f(x: List[object]) -> TypeGuard[List[str]]: ...
+    """, """
+      from typing import List
+
+      def f(x: List[object]) -> bool: ...
+    """)
+
+
 if __name__ == "__main__":
   unittest.main()
