@@ -129,10 +129,13 @@ class Typeshed:
       maximum = ((int(max_major), int(max_minor))
                  if max_major is not None and max_minor is not None
                  else None)
-      use_python2 = min_major == "2" and (
-          self._file_exists(os.path.join("stdlib", "@python2", module + ".pyi"))
-          or self._file_exists(os.path.join("stdlib", "@python2", module)))
-      versions[module] = minimum, maximum, use_python2
+      versions[module] = minimum, maximum, False
+    for path in self._list_files(os.path.join("stdlib", "@python2")):
+      mod, _ = os.path.splitext(path.split(os.path.sep, 1)[0])
+      if mod in versions:
+        (min_major, min_minor), maximum, _ = versions[mod]
+        if min_major == 2:
+          versions[mod] = (min_major, min_minor), maximum, True
     return versions
 
   def _load_third_party_packages(self):
@@ -140,33 +143,46 @@ class Typeshed:
 
     stubs/ contains type information for third-party packages. Each top-level
     directory corresponds to one PyPI package and contains one or more modules,
-    plus a metadata file (METADATA.toml). If there are separate Python 2 stubs,
-    they live in an @python2 subdirectory. METADATA.toml
-    takes @python2 into account, so if a package has both foo.pyi and
-    @python2/foo.pyi, METADATA.toml will contain `python2 = True`.
+    plus a metadata file (METADATA.toml). If a package supports Python 2,
+    it will either have separate stubs in a @python2 subdirectory or
+    the `python2 = true` flag is set in METADATA.toml. If a package supports
+    Python 3, it will have at least one module or package on the top level
+    and the `python3 = false` flag is not set in METADATA.toml.
 
     Returns:
       A mapping from module name to a set of
       (package name, major_python_version) tuples.
     """
-    metadata = {}
     modules = collections.defaultdict(set)
+    py2_dir = set()  # packages with @python2 directory
+    py2_meta = set()  # packages with `python2 = true` metadata entry
+    top_level_stubs = set()  # packages with stub files outside @python2
+    no_py3_meta = set()  # packages with `python3 = false` metadata entry
     for third_party_file in self._list_files("stubs"):
       parts = third_party_file.split(os.path.sep)
       if parts[-1] == "METADATA.toml":  # {package}/METADATA.toml
         _, metadata_file = self._load_file(
             os.path.join("stubs", third_party_file))
-        metadata[parts[0]] = toml.loads(metadata_file)
+        metadata = toml.loads(metadata_file)
+        if metadata.get("python2", False):
+          py2_meta.add(parts[0])
+        if not metadata.get("python3", True):
+          no_py3_meta.add(parts[0])
+      elif "@python2" in parts:  # {package}/@python2/{module}
+        py2_dir.add(parts[0])
+        name, _ = os.path.splitext(parts[2])
+        modules[parts[0]].add(name)
       elif "@python2" not in parts:  # {package}/{module}
+        if parts[-1].endswith(".pyi"):
+          top_level_stubs.add(parts[0])
         name, _ = os.path.splitext(parts[1])
         modules[parts[0]].add(name)
     packages = collections.defaultdict(set)
     for package, names in modules.items():
       for name in names:
-        # When not specified, packages are Python 3-only
-        if metadata[package].get("python2", False):
+        if package in py2_dir or package in py2_meta:
           packages[name].add((package, 2))
-        if metadata[package].get("python3", True):
+        if package in top_level_stubs and package not in no_py3_meta:
           packages[name].add((package, 3))
     return packages
 
