@@ -376,14 +376,21 @@ def _LookupItemRecursive(module, module_name, name):
   partial_name = module_name
   prev_item = None
   item = module
+
+  def ExtractClass(t):
+    if isinstance(t, pytd.ClassType):
+      return t.cls
+    t = module.Lookup(t.name)  # may raise KeyError
+    if isinstance(t, pytd.Class):
+      return t
+    raise KeyError(t.name)
+
   for part in parts:
     prev_item = item
     # Check the type of item and give up if we encounter a type we don't know
     # how to handle.
     if isinstance(item, pytd.Constant):
-      if not isinstance(item.type, pytd.ClassType):
-        raise KeyError(name)
-      item = item.type.cls
+      item = ExtractClass(item.type)  # may raise KeyError
     elif not isinstance(item, (pytd.TypeDeclUnit, pytd.Class)):
       raise KeyError(name)
     lookup_name = partial_name + "." + part
@@ -404,12 +411,9 @@ def _LookupItemRecursive(module, module_name, name):
       if not isinstance(item, pytd.Class):
         raise
       for parent in item.parents:
-        if not isinstance(parent, pytd.ClassType):
-          # If the parent is unknown, we don't know whether it contains the name
-          # part, so it cannot be resolved.
-          raise
+        parent_cls = ExtractClass(parent)  # may raise KeyError
         try:
-          item = Lookup(parent.cls, lookup_name, part)
+          item = Lookup(parent_cls, lookup_name, part)
         except KeyError:
           continue  # continue up the MRO
         else:
@@ -568,6 +572,7 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
     aliases = []
     getattrs = set()
     ast = self._module_map[module]
+    type_param_names = set()
     for member in sum((ast.constants, ast.type_params, ast.classes,
                        ast.functions, ast.aliases), ()):
       _, _, member_name = member.name.rpartition(".")
@@ -577,6 +582,12 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
         # than aliased.
         getattrs.add(member.Replace(name=new_name))
       else:
+        # Imported type parameters produce both a type parameter definition and
+        # an alias. Keep the definition and discard the alias.
+        if isinstance(member, pytd.TypeParameter):
+          type_param_names.add(new_name)
+        elif new_name in type_param_names:
+          continue
         t = pytd.ToType(member, allow_constants=True, allow_functions=True)
         aliases.append(pytd.Alias(new_name, t))
     return aliases, getattrs
@@ -1435,10 +1446,37 @@ class AdjustTypeParameters(Visitor):
   def LeaveSignature(self, unused_node):
     self.function_typeparams = None
 
+  def _MaybeMutateSelf(self, sig):
+    # If the given signature is an __init__ method for a generic class and the
+    # class's type parameters all appear among the method's parameter
+    # annotations, then we should add a mutation to the parameter values, e.g.:
+    #   class Foo(Generic[T]):
+    #      def __init__(self, x: T) -> None: ...
+    # becomes:
+    #   class Foo(Generic[T]):
+    #     def __init__(self, x: T) -> None:
+    #       self = Foo[T]
+    if self.function_name != "__init__" or not self.class_name:
+      return sig
+    class_template = self.class_template[-1]
+    if not class_template:
+      return sig
+    seen_params = {t.name: t for t in pytd_utils.GetTypeParameters(sig)}
+    if any(t.name not in seen_params for t in class_template):
+      return sig
+    if not sig.params or sig.params[0].mutated_type:
+      return sig
+    mutated_type = pytd.GenericType(
+        base_type=pytd.ClassType(self.class_name),
+        parameters=tuple(seen_params[t.name] for t in class_template))
+    self_param = sig.params[0].Replace(mutated_type=mutated_type)
+    return sig.Replace(params=(self_param,) + sig.params[1:])
+
   def VisitSignature(self, node):
     # Sorting the template in CanonicalOrderingVisitor is enough to guarantee
     # pyi determinism, but we need to sort here as well for pickle determinism.
-    return node.Replace(template=tuple(sorted(self.function_typeparams)))
+    return self._MaybeMutateSelf(
+        node.Replace(template=tuple(sorted(self.function_typeparams))))
 
   def EnterFunction(self, node):
     self.function_name = node.name
@@ -1528,13 +1566,22 @@ class VerifyContainers(Visitor):
       return  # We can't verify this yet
     if not pytd.IsContainer(base_type.cls):
       raise ContainerError("Class %s is not a container" % base_type.name)
-    elif base_type.name == "typing.Generic":
+    elif base_type.name in ("typing.Generic", "typing.Protocol"):
       for t in node.parameters:
         if not isinstance(t, pytd.TypeParameter):
           raise ContainerError("Name %s must be defined as a TypeVar" % t.name)
     elif not isinstance(node, (pytd.CallableType, pytd.TupleType)):
-      max_param_count = len(base_type.cls.template)
       actual_param_count = len(node.parameters)
+      if actual_param_count and not base_type.cls.template:
+        # This AdjustTypeParameters() call is needed because we validate nodes
+        # before their type parameters have been adjusted in some circular
+        # import cases. The result of this adjustment is not saved because it
+        # may not be accurate if the container is only partially resolved, but
+        # it's good enough to avoid some spurious container validation errors.
+        cls = base_type.cls.Visit(AdjustTypeParameters())
+      else:
+        cls = base_type.cls
+      max_param_count = len(cls.template)
       if actual_param_count > max_param_count:
         raise ContainerError(
             "Too many parameters on %s: expected %s, got %s" % (
@@ -1814,16 +1861,3 @@ class DropMutableParameters(Visitor):
 
   def VisitParameter(self, p):
     return p.Replace(mutated_type=None)
-
-
-class ClassTypeToAny(base_visitor.Visitor):
-  """Change all ClassType objects with a given name to Any."""
-
-  def __init__(self, target_name):
-    super().__init__()
-    self.target_name = target_name
-
-  def VisitClassType(self, node):
-    if node.name == self.target_name:
-      return pytd.AnythingType()
-    return node
