@@ -2,6 +2,7 @@
 
 import collections
 import logging
+from typing import Collection, Optional
 
 from pytype import abstract
 from pytype import class_mixin
@@ -233,7 +234,8 @@ class Frame(utils.VirtualMachineWeakrefMixin):
   """
 
   def __init__(self, node, vm, f_code, f_globals, f_locals, f_back, callargs,
-               closure, func, first_arg=None):
+               closure, func, first_arg: Optional[cfg.Variable],
+               type_params: Collection[abstract.TypeParameter]):
     """Initialize a special frame as needed by TypegraphVirtualMachine.
 
     Args:
@@ -247,7 +249,8 @@ class Frame(utils.VirtualMachineWeakrefMixin):
       callargs: Additional function arguments to store in f_locals.
       closure: A tuple containing the new co_freevars.
       func: An Optional[cfg.Binding] to the function this frame corresponds to.
-      first_arg: Optional first argument to the function.
+      first_arg: First argument to the function.
+      type_params: Type parameters in scope for this frame.
     Raises:
       NameError: If we can't resolve any references into the outer frame.
     """
@@ -320,6 +323,7 @@ class Frame(utils.VirtualMachineWeakrefMixin):
         i = f_code.co_cellvars.index(closure_name)
         self.class_closure_var = self.cells[i]
     self.func = func
+    self.type_params = type_params
 
   def __repr__(self):     # pragma: no cover
     return "<Frame at 0x%08x: %r @ %d>" % (
