@@ -63,12 +63,16 @@ ITEMS = {
 _PYTYPE_SINGLE_ITEMS = {
     'attribute_variable_annotations': Item(
         None, 'False', ArgInfo('--attribute-variable-annotations', None), None),
+    'bind_properties': Item(
+        None, 'False', ArgInfo('--bind-properties', None), None),
     'check_parameter_types': Item(
         None, 'False', ArgInfo('--check-parameter-types', None), None),
     'create_pyi_dataclasses': Item(
         None, 'False', ArgInfo('--create-pyi-dataclasses', None), None),
     'preserve_union_macros': Item(
         None, 'False', ArgInfo('--preserve-union-macros', None), None),
+    'use_enum_overlay': Item(
+        None, 'False', ArgInfo('--use-enum-overlay', None), None),
     'disable': Item(
         None, 'pyi-error', ArgInfo('--disable', ','.join),
         'Comma or space separated list of error names to ignore.'),
