@@ -143,6 +143,12 @@ class TestCustomOptions(TestBase):
     super().setUp()
     self.conf = self.parser.config_from_defaults()
 
+  def assertFlags(self, flags, expected_flags):
+    # Add temporary flags that are set to true by default here, so that they are
+    # filtered out of tests.
+    temporary_flags = set()
+    self.assertEqual(flags - temporary_flags, expected_flags)
+
   # --disable tests a flag with a string value.
 
   def test_disable(self):
@@ -166,21 +172,21 @@ class TestCustomOptions(TestBase):
     runner = make_runner([], [], self.conf)
     binary_flags = {'--no-report-errors'}
     runner.set_custom_options({}, binary_flags)
-    self.assertEqual(binary_flags, set())
+    self.assertFlags(binary_flags, set())
 
   def test_no_report_errors(self):
     self.conf.report_errors = False
     runner = make_runner([], [], self.conf)
     binary_flags = set()
     runner.set_custom_options({}, binary_flags)
-    self.assertEqual(binary_flags, {'--no-report-errors'})
+    self.assertFlags(binary_flags, {'--no-report-errors'})
 
   def test_report_errors_default(self):
     self.conf.report_errors = True
     runner = make_runner([], [], self.conf)
     binary_flags = set()
     runner.set_custom_options({}, binary_flags)
-    self.assertEqual(binary_flags, set())
+    self.assertFlags(binary_flags, set())
 
   # --protocols tests a binary flag whose value is passed through transparently.
 
@@ -189,21 +195,21 @@ class TestCustomOptions(TestBase):
     runner = make_runner([], [], self.conf)
     binary_flags = set()
     runner.set_custom_options({}, binary_flags)
-    self.assertEqual(binary_flags, {'--protocols'})
+    self.assertFlags(binary_flags, {'--protocols'})
 
   def test_no_protocols(self):
     self.conf.protocols = False
     runner = make_runner([], [], self.conf)
     binary_flags = {'--protocols'}
     runner.set_custom_options({}, binary_flags)
-    self.assertEqual(binary_flags, set())
+    self.assertFlags(binary_flags, set())
 
   def test_no_protocols_default(self):
     self.conf.protocols = False
     runner = make_runner([], [], self.conf)
     binary_flags = set()
     runner.set_custom_options({}, binary_flags)
-    self.assertEqual(binary_flags, set())
+    self.assertFlags(binary_flags, set())
 
 
 class TestGetRunCmd(TestBase):
