@@ -3,12 +3,13 @@
 import collections
 
 import attr
+from pytype.pytd import escape
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 
 
 @attr.s
-class Attr(object):
+class Attr:
   name = attr.ib()
   node_type = attr.ib()
   type = attr.ib()
@@ -17,27 +18,27 @@ class Attr(object):
 
 
 @attr.s
-class Arg(object):
+class Arg:
   name = attr.ib()
   node_type = attr.ib()
   type = attr.ib()
 
 
 @attr.s
-class Param(object):
+class Param:
   name = attr.ib()
   type = attr.ib()
 
 
 @attr.s
-class Call(object):
+class Call:
   function_id = attr.ib()
   args = attr.ib()
   location = attr.ib()
 
 
 @attr.s
-class Function(object):
+class Function:
   id = attr.ib()
   params = attr.ib(factory=list)
   param_attrs = attr.ib(factory=list)
@@ -48,7 +49,7 @@ class Function(object):
 
 
 def unknown_to_any(typename):
-  if '~unknown' in typename:
+  if escape.UNKNOWN in typename:
     return 'typing.Any'
   return typename
 
@@ -76,14 +77,14 @@ def get_function_params(pytd_fn):
     for p in sig.params:
       if p.name not in params:
         params[p.name] = []
-      if '~unknown' not in str(p.type):
+      if escape.UNKNOWN not in str(p.type):
         params[p.name].append(p.type)
   for k in params:
     params[k] = pytd_utils.JoinTypes(params[k])
   return [(k, unwrap_type(v)) for k, v in params.items()]
 
 
-class FunctionMap(object):
+class FunctionMap:
   """Collect a map of function types and outbound callgraph edges."""
 
   def __init__(self, index):
